#!/usr/bin/python3

RETURN = '''# '''

from subprocess import Popen, PIPE, STDOUT


def main():
    module = AnsibleModule(
        argument_spec=dict(
            mode=dict(required=False, default='check'),
            executable=dict(required=False, default='pip'),
            lsb_release=dict(required=False, default='16'),
        ),
        supports_check_mode=False
    )
    constraints = {
        'dummy': 'v.e.r',
        'colorama': '0.3.7',
        'docutils': '0.15.2',# botocore requires <16
        'msgpack': '0.5.6',# Required for BorgBackup
        'chardet': '3.0.4',# Certbot fails with 4.0.0
        'idna': '2.10',# Certbot fails with 3.0
    }
    mode = module.params['mode']
    executable = module.params['executable']
    lsb_release = module.params['lsb_release']
    if lsb_release == '16':
        constraints['setuptools'] = '49.0.0'
    result = []

    changed = (lsb_release == '16') or (lsb_release == '18')
    if changed:
        process = Popen([executable, 'list', '--outdated', '--format=json'], stdout=PIPE, stderr=STDOUT)
        process.wait()
        updates = []
        for line in process.stdout.read().decode('utf8').split("\n"):
            if line != '' and line[0] == '[':
                updates = json.loads(line)
        for item in updates:
            if item['name'] in constraints and constraints[item['name']] == item['version']:
                # This doesn't need an update
                pass
            else:
                result.append(item)

    if len(result) == 0:
        module.exit_json(changed=False, result="No updates available")

    if changed and mode == 'upgrade':
        args = [executable, 'install', '--upgrade']
        if len(constraints) > 0:
            args.append('--constraint=/tmp/ansible-pip-constraints')
            with open('/tmp/ansible-pip-constraints', "w") as file:
                for constraint in constraints:
                    file.write("%s==%s\n" % (constraint, constraints[constraint]))
        for item in result:
            args.append(item['name'])
        process = Popen(args, stdout=PIPE, stderr=PIPE)
        process.wait()
        msg = "%s\n\n%s" % (process.stdout.read().decode('utf8'), process.stderr.read().decode('utf8'))
    else:
        msg = "Available updates:\n"
        for item in result:
            msg += "%s %s (installed: %s)\n" % (item['name'], item['latest_version'], item['version'])

    module.exit_json(changed=changed, result=msg)


from ansible.module_utils.basic import *

if __name__ == '__main__':
    main()
